﻿using System;
using System.IO;
using System.Data;
using System.Drawing;

namespace BMPViewer
{
    class Car_Palte
    {
        Color[,] Image_Data;
        BMPViewerForm BMPImage;

        public Car_Palte(BMPViewerForm BMP, Color[,] Data)
        {
            BMPImage = new BMPViewerForm();
            BMPImage = BMP;
            Image_Data = new Color[BMP.image_Height, BMP.image_Width];
            Image_Data = Data;
        }

        public Color[,] Detect_Plate(out int outCount, float valve, out int outxu, out int outxd, out int outyl, out int outyr)
        {
            Color[,] Data = new Color[BMPImage.image_Height, BMPImage.image_Width];
            int Sx = 0;
            int Sy = 0;

            int[,] marginalMx = new int[,] { { -1, 0, 1 }, { -2, 0, 2 }, { -1, 0, 1 } };
            int[,] marginalMy = new int[,] { { 1, 2, 1 }, { 0, 0, 0 }, { -1, -2, -1 } };

            float[,] m = new float[BMPImage.image_Height, BMPImage.image_Width];


            for (int y = 0; y < BMPImage.image_Height; ++y)
            {
                for (int x = 0; x < BMPImage.image_Width; ++x)
                {
                    Sx = 0;
                    Sy = 0;
                    for (int i = 0; i < 3; i++)
                    {
                        for (int j = 0; j < 3; j++)
                        {
                            if (x - 1 + j < 0 || y - 1 + i < 0 || x - 1 + j >= BMPImage.image_Width || y - 1 + i >= BMPImage.image_Height)
                            {
                                m[y, x] = 0;
                            }

                            else
                            {
                                Sx += Convert.ToInt32(Image_Data[y - 1 + i, x - 1 + j].R.ToString()) * marginalMx[i, j];
                                Sy += Convert.ToInt32(Image_Data[y - 1 + i, x - 1 + j].R.ToString()) * marginalMy[i, j];
                            }
                        }
                    }

                    m[y, x] = (int)(Math.Sqrt(Sx * Sx + Sy * Sy));

                    Data[y, x] = Color.FromArgb(Math.Min(255, (int)m[y, x]), Math.Min(255, (int)m[y, x]), Math.Min(255, Math.Min(255, (int)m[y, x])));
                }
            }

            int[] Px = new int[BMPImage.image_Width];
            int[] Px1 = new int[BMPImage.image_Width];
            int[] Py = new int[BMPImage.image_Height];
            int[] Py1 = new int[BMPImage.image_Height];

            Py1 = calc_VProjection(Image_Data);
            Py = decrease_ragged(Py1);

            Px1 = calc_HProjection(Image_Data);
            Px = decrease_ragged(Px1);

            int xp0 = 0, xb0 = xp0, xp1 = BMPImage.image_Width - 1, xb1 = xp1, xbm = 1;
            int yp0 = 0, yb0 = yp0, yp1 = BMPImage.image_Height - 1, yb1 = yp1, ybm = 1;

            for (int i = 1; i < BMPImage.image_Height - 1; i++)
            {
                if (Py[i] > Py[ybm])
                    ybm = i;
            }

            for (int i = 1; i <= ybm; i++)
            {
                if (Py[i] <= 0.55 * Py[ybm])
                    if (i > yb0)
                        yb0 = i;
            }

            for (int i = ybm; i < BMPImage.image_Height - 1; i++)
            {
                if (Py[i] <= 0.42 * Py[ybm])
                    if (i < yb1)
                        yb1 = i;
            }

            ////////////////////

            for (int i = yb0; i < yb1; i++)
            {
                if (Px[i] > Py[xbm])
                    xbm = i;
            }

            for (int i = 1; i <= xbm; i++)
            {
                if (Px[i] <= 0.86 * Px[xbm])
                    if (i > xb0)
                        xb0 = i;
            }

            for (int i = xbm; i < BMPImage.image_Width - 1; i++)
            {
                if (Px[i] <= 0.86 * Px[xbm])
                    if (xb1 > i)
                        xb1 = i;
            }

            outxd = yb1;
            outxu = yb0;
            outyl = xb0;
            outyr = xb1;

            outCount = 10;
            return Data;
        }

        int[] calc_VProjection(Color[,] Image)
        {
            int[,] Filtered = new int[BMPImage.image_Height, BMPImage.image_Width];
            int[] PVertical = new int[BMPImage.image_Height];

            int[,] Filter = new int[,] { { -1, 0, 1 }, { -1, 0, 1 }, { -1, 0, 1 } };

            for (int y = 0; y < BMPImage.image_Height; ++y)
            {
                for (int x = 0; x < BMPImage.image_Width; ++x)
                {
                    int Pixel = 0;
                    for (int i = 0; i < 3; i++)
                    {
                        for (int j = 0; j < 3; j++)
                        {
                            if (x - 1 + j < 0 || y - 1 + i < 0 || x - 1 + j >= BMPImage.image_Width || y - 1 + i >= BMPImage.image_Height)
                            {
                                Pixel += 0;
                            }

                            else
                            {
                                Pixel += Image[y - 1 + i, x - 1 + j].R * Filter[i, j];
                            }
                        }
                    }

                    Filtered[y, x] = Math.Abs(Pixel);
                }
            }

            for (int y = 0; y < BMPImage.image_Height; ++y)
            {
                PVertical[y] = 0;
                for (int x = 0; x < BMPImage.image_Width; ++x)
                {
                    PVertical[y] += Filtered[y, x];
                }
            }

            return PVertical;
        }

        int[] calc_HProjection(Color[,] Image)
        {
            int[,] Filtered = new int[BMPImage.image_Height, BMPImage.image_Width];
            int[] PHorizontal = new int[BMPImage.Width];

            int[,] Filter = new int[,] { { -1, -1, -1 }, { 0, 0, 0 }, { 1, 1, 1 } };

            for (int y = 0; y < BMPImage.image_Height; ++y)
            {
                for (int x = 0; x < BMPImage.image_Width; ++x)
                {
                    int Pixel = 0;
                    for (int i = 0; i < 3; i++)
                    {
                        for (int j = 0; j < 3; j++)
                        {
                            if (x - 1 + j < 0 || y - 1 + i < 0 || x - 1 + j >= BMPImage.image_Width || y - 1 + i >= BMPImage.image_Height)
                            {
                                Pixel += 0;
                            }

                            else
                            {
                                Pixel += Convert.ToInt32(Image[y - 1 + i, x - 1 + j].R.ToString()) * Filter[i, j];
                            }
                        }
                    }

                    Filtered[y, x] = Math.Abs(Pixel);
                }
            }

            for (int x = 0; x < BMPImage.image_Width; ++x)
            {
                PHorizontal[x] = 0;
                for (int y = 0; y < BMPImage.image_Height; ++y)
                {
                    PHorizontal[x] += Filtered[y, x];
                }
            }

            return PHorizontal;
        }

        public int[] decrease_ragged(int[] Matrix)
        {
            double Filter = 1.0 / 9.0;
            int[] Filtered = new int[Matrix.Length];
            for (int i = 0; i < Matrix.Length; i++)
            {
                double[] Value = new double[9];

                for (int j = -4; j < 4; j++)
                {
                    if (i + j < 0 || i + j > Matrix.Length - 1)
                        Value[j + 4] = 0;

                    else
                    {
                        Value[j + 4] = Matrix[i + j];
                    }
                }

                Array.Sort(Value);

                Filtered[i] = (int)Value[4];
            }

            return Filtered;
        }
    }
}
