﻿using System;
using System.IO;
using System.Data;
using System.Drawing;

namespace BMPViewer
{
    class Filter
    {
        Color[,] Image_Data;
        BMPViewerForm BMPImage;

        public Filter(BMPViewerForm BMP, Color[,] Data)
        {
            BMPImage = new BMPViewerForm();
            BMPImage = BMP;
            Image_Data = new Color[BMP.image_Height, BMP.image_Width];
            Image_Data = Data;
        }

        public Color[,] Average1_Filter()
        {
            Color[,] Filterd_Data = new Color[BMPImage.image_Height, BMPImage.image_Width];
            int[,] Filter_Matrix = new int[,] { { 1, 1, 1 }, { 1, 1, 1 }, { 1, 1, 1 } };
            int n = 9;
            Filterd_Data = Average(Filter_Matrix, n);

            return Filterd_Data;
        }

        public Color[,] Average2_Filter()
        {
            Color[,] Filterd_Data = new Color[BMPImage.image_Height, BMPImage.image_Width];
            int[,] Filter_Matrix = new int[,] { { 1, 2, 1 }, { 2, 4, 2 }, { 1, 2, 1 } };
            int n = 16;
            Filterd_Data = Average(Filter_Matrix, n);

            return Filterd_Data;
        }

        public Color[,] Median_Filter()
        {
            Color[,] Filterd_Data = new Color[BMPImage.image_Height, BMPImage.image_Width];
            int[,] Filter_Matrix = new int[,] { { 1, 1, 1 }, { 1, 1, 1 }, { 1, 1, 1 } };
            int n = 9;
            Filterd_Data = Medium(Filter_Matrix, n);

            return Filterd_Data;
        }

        public Color[,] Average(int[,] Filter_Matrix, int num)
        {
            Color[,] Filterd_Data = new Color[BMPImage.image_Height, BMPImage.image_Width];

            int Pixel1, Pixel2, Pixel3;
            double Total1, Total2, Total3;
            for (int i = 0; i < BMPImage.image_Height; i++)
            {
                for (int j = 0; j < BMPImage.image_Width; j++)
                {
                    Total1 = Total2 = Total3 = 0;
                    for (int k = 0; k < 3; k++)
                    {
                        for (int l = 0; l < 3; l++)
                        {
                            if (j - 1 + l < 0 || i - 1 + k < 0 || j - 1 + l >= BMPImage.image_Width || i - 1 + k >= BMPImage.image_Height)
                                Pixel1 = Pixel2 = Pixel3 = 0;
                            else
                            {
                                Pixel1 = Convert.ToInt32(Image_Data[i - 1 + k, j - 1 + l].R.ToString());
                                Pixel2 = Convert.ToInt32(Image_Data[i - 1 + k, j - 1 + l].G.ToString());
                                Pixel3 = Convert.ToInt32(Image_Data[i - 1 + k, j - 1 + l].B.ToString());
                            }
                            Total1 += Pixel1 * Filter_Matrix[l, k];
                            Total2 += Pixel2 * Filter_Matrix[l, k];
                            Total3 += Pixel3 * Filter_Matrix[l, k];
                        }
                    }
                    Filterd_Data[i, j] = Color.FromArgb(Convert.ToInt32(Total1 / num), Convert.ToInt32(Total2 / num), Convert.ToInt32(Total3 / num));
                }
            }
            return Filterd_Data;
        }

        public Color[,] Medium(int[,] Filter_Matrix, int num)
        {
            Color[,] Filterd_Data = new Color[BMPImage.image_Height, BMPImage.image_Width];

            int Pixel1, Pixel2, Pixel3;
            int[] Total1, Total2, Total3;
            Total1 = new int[9];
            Total2 = new int[9];
            Total3 = new int[9];

            for (int i = 0; i < BMPImage.image_Height; i++)
            {
                for (int j = 0; j < BMPImage.image_Width; j++)
                {
                    for (int k = 0; k < 3; k++)
                    {
                        for (int l = 0; l < 3; l++)
                        {
                            if (j - 1 + l < 0 || i - 1 + k < 0 || j - 1 + l >= BMPImage.image_Width || i - 1 + k >= BMPImage.image_Height)
                            {
                                Pixel1 = Pixel2 = Pixel3 = 0;
                            }

                            else
                            {
                                Pixel1 = Convert.ToInt32(Image_Data[i - 1 + k, j - 1 + l].R.ToString());
                                Pixel2 = Convert.ToInt32(Image_Data[i - 1 + k, j - 1 + l].G.ToString());
                                Pixel3 = Convert.ToInt32(Image_Data[i - 1 + k, j - 1 + l].B.ToString());
                            }
                            Total1[3 * k + l] = Pixel1 * Filter_Matrix[l, k];
                            Total2[3 * k + l] = Pixel2 * Filter_Matrix[l, k];
                            Total3[3 * k + l] = Pixel3 * Filter_Matrix[l, k];
                        }
                    }
                    int med1 = Calc_Medium(Total1);
                    int med2 = Calc_Medium(Total2);
                    int med3 = Calc_Medium(Total3);
                    Filterd_Data[i, j] = Color.FromArgb(Total1[med1], Total2[med2], Total3[med3]);
                }
            }
            return Filterd_Data;
        }

        public int Calc_Medium(int[] Arr)
        {
            Array.Sort(Arr);
            int Med;
            
            Med = (Arr.Length - 1) / 2;
            return Med;
        }
    }
}
