﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace BMPViewer
{
    public partial class PlateNumber : Form
    {
        Bitmap Plate;
        static Color[,] Data;
        static double[] Image_Hist;

        public PlateNumber(Bitmap P)
        {
            InitializeComponent();
            Plate = new Bitmap(P.Width, P.Height);
            Plate = P;
            Data = new Color[P.Height, P.Width];
            Image_Hist = new double[256];
        }

        private void Car_Plate_Load(object sender, EventArgs e)
        {
            for (int i = 0; i < Plate.Height; i++)
            {
                for (int j = 0; j < Plate.Width; j++)
                {
                    Data[i, j] = Plate.GetPixel(j, i);
                }
            }

            CarPlate.Location = new Point(12 + (258 - Plate.Width) / 2, 12 + (128 - Plate.Height) / 2);
            CarPlate.Size = new Size(Plate.Width, Plate.Height);
            CarPlate.Image = Plate;
            CarPlate.Visible = true;

            Bitmap Segmented_BMP = Segmentation();

            Segmented_Plate.Location = new Point(309 + (258 - Plate.Width) / 2, 12 + (128 - Plate.Height) / 2);
            Segmented_Plate.Size = new Size(Plate.Width, Plate.Height);
            Segmented_Plate.Image = Segmented_BMP;
            Segmented_Plate.Visible = true;
        }

        Bitmap Segmentation()
        {
            Bitmap BMPIMG = new Bitmap(Plate.Width, Plate.Height);
            
            for (int i = 0; i < Plate.Height; i++)
            {
                for (int j = 0; j < Plate.Width; j++)
                {
                    Image_Hist[Data[i, j].R]++;
                }
            }

            Data = Thresholding(Image_Hist);
            int[,] labels = new int[Plate.Height, Plate.Width];
            int number;

            labels = LabelImage(Plate, out number);
            int count = 0;

            for (int l = 1; l <= number; l++)
            {
                int xmin = Plate.Width, ymin = Plate.Height, xmax = 0, ymax = 0;
                int counter = 0;

                for (int i = 0; i < Plate.Height; i++)
                {
                    for (int j = 0; j < Plate.Width; j++)
                    {
                        if (labels[i, j] == l)
                        {
                            counter++;

                            if (j < xmin)
                                xmin = j;

                            if (i < ymin)
                                ymin = i;

                            if (j > xmax)
                                xmax = j;

                            if (i > ymax)
                                ymax = i;
                        }
                    }
                }

                if (counter < 40)
                    continue;

                Bitmap Segment = new Bitmap(xmax - xmin, ymax - ymin);

                count++;

                for (int i = ymin; i < ymax; i++)
                {
                    for (int j = xmin; j < xmax; j++)
                    {
                        if (labels[i, j] == l)
                        {
                            Segment.SetPixel(j - (xmin), i - (ymin), Color.Black);
                        }

                        else
                            Segment.SetPixel(j - (xmin), i - (ymin), Color.White);
                    }
                }

                Segment.Save("Segment " + count.ToString() + ".bmp", System.Drawing.Imaging.ImageFormat.Bmp);
            }

            for (int i = 0; i < Plate.Height; i++)
            {
                for (int j = 0; j < Plate.Width; j++)
                {
                    int newlabel = (labels[i, j] * 100 + 50) % 255;

                    if (Data[i, j] == Color.Black)
                    {
                        Data[i, j] = Color.FromArgb(Math.Abs(newlabel + 30) % 255, Math.Abs(newlabel + 10) % 255, Math.Abs(newlabel + 50) % 255);
                        BMPIMG.SetPixel(j, i, Data[i, j]);
                    }

                    else
                        BMPIMG.SetPixel(j, i, Color.White);
                }
            }

            MessageBox.Show("Total Number of Objects in the Image = " + number.ToString() + "\nNumber of real Objects = " + count.ToString()
                            , "Hint", MessageBoxButtons.OK, MessageBoxIcon.Information);
            return BMPIMG;
        }

        public Color[,] Thresholding(double[] Hist)
        {
            int T = Threshold_Op();

            for (int i = 0; i < Plate.Height; i++)
            {
                for (int j = 0; j < Plate.Width; j++)
                {
                    if (Convert.ToInt32(Data[i, j].R.ToString()) >= T)
                        Data[i, j] = Color.White;
                    else
                        Data[i, j] = Color.Black;
                }
            }
            return Data;
        }

        private int Threshold_Op()
        {
            int T;
            double[] Variance_Arr = new double[256];
            double WB, WO, MeanB, MeanO, TotalB, TotalO;

            for (int i = 0; i < 256; i++)
            {
                WB = WO = MeanB = MeanO = TotalB = TotalO = 0;
                for (int j = 0; j < i; j++)
                {
                    WB += Image_Hist[j] / (Plate.Height * Plate.Width);
                    TotalB += Image_Hist[j];
                    MeanB += j * Image_Hist[j];
                }
                MeanB = (TotalB == 0) ? 0 : MeanB / TotalB;

                for (int k = i; k < 256; k++)
                {
                    WO += Image_Hist[k] / (Plate.Height * Plate.Width);
                    TotalO += Image_Hist[k];
                    MeanO += k * Image_Hist[k];
                }
                MeanO = (TotalO == 0) ? 0 : MeanO / TotalO;

                Variance_Arr[i] = WB * WO * Math.Pow((MeanO - MeanB), 2);
            }
            T = MaxIndex(Variance_Arr);
            return T;
        }

        private int MaxIndex(double[] Arr)
        {
            double Max = Arr[0];
            int index = 0;
            for (int i = 1; i < Arr.Length - 1; i++)
            {
                if (Arr[i] > Max)
                {
                    Max = Arr[i];
                    index = i;
                }
            }
            return index;
        }

        private int[,] LabelImage(Bitmap b, out int num)
        {
            int[,] label = new int[b.Height, b.Width];

            for (int i = 0; i != b.Height; i++)
            {
                for (int j = 0; j != b.Width; j++)
                {
                    if (Data[i, j] == Color.Black)
                        label[i, j] = -1;
                }
            }

            int lab = 1;
            int[] pos;
            Stack<int[]> stack = new Stack<int[]>();

            for (int r = 0; r < b.Height; r++)
            {
                for (int c = 0; c < b.Width; c++)
                {
                    if (Data[r, c] == Color.White || label[r, c] != -1)
                        continue;

                    stack.Push(new int[] { r, c });
                    label[r, c] = lab;

                    while (stack.Count != 0)
                    {
                        pos = (int[])stack.Pop();
                        int i = pos[0];
                        int j = pos[1];

                        if (i > 0 && j > 0)
                            if (Data[i - 1, j - 1] == Color.Black && label[i - 1, j - 1] == -1)
                            {
                                stack.Push(new int[] { i - 1, j - 1 });
                                label[i - 1, j - 1] = lab;
                            }

                        if (i > 0)
                            if (Data[i - 1, j] == Color.Black && label[i - 1, j] == -1)
                            {
                                stack.Push(new int[] { i - 1, j });
                                label[i - 1, j] = lab;
                            }

                        if (i > 0 && j < b.Width - 1)
                            if (Data[i - 1, j + 1] == Color.Black && label[i - 1, j + 1] == -1)
                            {
                                stack.Push(new int[] { i - 1, j + 1 });
                                label[i - 1, j + 1] = lab;
                            }

                        if (j > 0)
                            if (Data[i, j - 1] == Color.Black && label[i, j - 1] == -1)
                            {
                                stack.Push(new int[] { i, j - 1 });
                                label[i, j - 1] = lab;
                            }

                        if (j < b.Width - 1)
                            if (Data[i, j + 1] == Color.Black && label[i, j + 1] == -1)
                            {
                                stack.Push(new int[] { i, j + 1 });
                                label[i, j + 1] = lab;
                            }

                        if (i < b.Height - 1 && j > 0)
                            if (Data[i + 1, j - 1] == Color.Black && label[i + 1, j - 1] == -1)
                            {
                                stack.Push(new int[] { i + 1, j - 1 });
                                label[i + 1, j - 1] = lab;
                            }

                        if (i < b.Height - 1)
                            if (Data[i + 1, j] == Color.Black && label[i + 1, j] == -1)
                            {
                                stack.Push(new int[] { i + 1, j });
                                label[i + 1, j] = lab;
                            }

                        if (i < b.Height - 1 && j < b.Width - 1)
                            if (Data[i + 1, j + 1] == Color.Black && label[i + 1, j + 1] == -1)
                            {
                                stack.Push(new int[] { i + 1, j + 1 });
                                label[i + 1, j + 1] = lab;
                            }
                    }
                    lab++;
                }
            }

            num = --lab;
            return label;
        }
    }
}
